@@ -1,5 +1,6 @@
-/* Convert calendar time to simple time, inverse of mktime.
-   Copyright (C) 2003 Free Software Foundation, Inc.
+/* Convert UTC calendar time to simple time.  Like mktime but assumes UTC.
+
+   Copyright (C) 2003, 2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,19 +14,10 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
-#ifdef HAVE_TIMEGM
-
-/* Get timegm declaration.  */
 #include <time.h>
 
-#else
-
-/* Get time_t and struct tm */
-#include <time.h>
-
-/* Convert calendar time to simple time, inverse of mktime. */
-extern time_t timegm (struct tm *tm);
-
+#if defined HAVE_DECL_TIMEGM && !HAVE_DECL_TIMEGM
+time_t timegm (struct tm *tm);
 #endif
