@@ -1,5 +1,5 @@
 /* encapreppart.c --- Encrypted authentication reply part functions.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2006  Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -40,9 +40,12 @@ shishi_encapreppart (Shishi * handle)
   int res;
   Shishi_asn1 node = NULL;
   struct timeval tv;
-  struct timezone tz;
   uint32_t seqnr;
 
+  res = gettimeofday (&tv, NULL);
+  if (res != 0)
+    return NULL;
+
   node = shishi_asn1_encapreppart (handle);
   if (!node)
     return NULL;
@@ -52,7 +55,6 @@ shishi_encapreppart (Shishi * handle)
   if (res != SHISHI_OK)
     goto error;
 
-  gettimeofday (&tv, &tz);
   res = shishi_encapreppart_cusec_set (handle, node, tv.tv_usec % 1000000);
   if (res != SHISHI_OK)
     goto error;
