@@ -1,5 +1,5 @@
 /* krberror.c --- Functions related to KRB-ERROR packet.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2006  Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -21,9 +21,6 @@
 
 #include "internal.h"
 
-/* Get xgettimeofday. */
-#include "utils.h"
-
 /* Get _shishi_print_armored_data, etc. */
 #include "diskio.h"
 
@@ -46,10 +43,11 @@ shishi_krberror (Shishi * handle)
 {
   Shishi_asn1 krberror;
   struct timeval tv;
-  struct timezone tz;
   int rc;
 
-  xgettimeofday (&tv, &tz);
+  rc = gettimeofday (&tv, NULL);
+  if (rc != 0)
+    return NULL;
 
   krberror = shishi_asn1_krberror (handle);
   if (!krberror)
@@ -1171,88 +1169,144 @@ struct krb_error_msgs
   const char *message;
 };
 
-struct krb_error_msgs _shishi_krberror_messages[SHISHI_LAST_ERROR_CODE] = {
-  {SHISHI_KDC_ERR_NONE, "No error"},
-  {SHISHI_KDC_ERR_NAME_EXP, "Client's entry in database has expired"},
-  {SHISHI_KDC_ERR_SERVICE_EXP, "Server's entry in database has expired"},
+static const struct krb_error_msgs
+_shishi_krberror_messages[SHISHI_LAST_ERROR_CODE] = {
+  {SHISHI_KDC_ERR_NONE,
+   N_("No error")},
+  {SHISHI_KDC_ERR_NAME_EXP,
+   N_("Client's entry in database has expired")},
+  {SHISHI_KDC_ERR_SERVICE_EXP,
+   N_("Server's entry in database has expired")},
   {SHISHI_KDC_ERR_BAD_PVNO,
-   "Requested protocol version number not supported"},
+   N_("Requested protocol version number not supported")},
   {SHISHI_KDC_ERR_C_OLD_MAST_KVNO,
-   "Client's key encrypted in old master key"},
+   N_("Client's key encrypted in old master key")},
   {SHISHI_KDC_ERR_S_OLD_MAST_KVNO,
-   "Server's key encrypted in old master key"},
+   N_("Server's key encrypted in old master key")},
   {SHISHI_KDC_ERR_C_PRINCIPAL_UNKNOWN,
-   "Client not found in database"},
+   N_("Client not found in database")},
   {SHISHI_KDC_ERR_S_PRINCIPAL_UNKNOWN,
-   "Server not found in database"},
+   N_("Server not found in database")},
   {SHISHI_KDC_ERR_PRINCIPAL_NOT_UNIQUE,
-   "Multiple principal entries in database"},
-  {SHISHI_KDC_ERR_NULL_KEY, "The client or server has a null key"},
-  {SHISHI_KDC_ERR_CANNOT_POSTDATE, "Ticket not eligible for postdating"},
-  {SHISHI_KDC_ERR_NEVER_VALID, "Requested start time is later than end time"},
-  {SHISHI_KDC_ERR_POLICY, "KDC policy rejects request"},
-  {SHISHI_KDC_ERR_BADOPTION, "KDC cannot accommodate requested option"},
-  {SHISHI_KDC_ERR_ETYPE_NOSUPP, "KDC has no support for encryption type"},
-  {SHISHI_KDC_ERR_SUMTYPE_NOSUPP, "KDC has no support for checksum type"},
-  {SHISHI_KDC_ERR_PADATA_TYPE_NOSUPP, "KDC has no support for padata type"},
-  {SHISHI_KDC_ERR_TRTYPE_NOSUPP, "KDC has no support for transited type"},
-  {SHISHI_KDC_ERR_CLIENT_REVOKED, "Clients credentials have been revoked"},
+   N_("Multiple principal entries in database")},
+  {SHISHI_KDC_ERR_NULL_KEY,
+   N_("The client or server has a null key")},
+  {SHISHI_KDC_ERR_CANNOT_POSTDATE,
+   N_("Ticket not eligible for postdating")},
+  {SHISHI_KDC_ERR_NEVER_VALID,
+   N_("Requested start time is later than end time")},
+  {SHISHI_KDC_ERR_POLICY,
+   N_("KDC policy rejects request")},
+  {SHISHI_KDC_ERR_BADOPTION,
+   N_("KDC cannot accommodate requested option")},
+  {SHISHI_KDC_ERR_ETYPE_NOSUPP,
+   N_("KDC has no support for encryption type")},
+  {SHISHI_KDC_ERR_SUMTYPE_NOSUPP,
+   N_("KDC has no support for checksum type")},
+  {SHISHI_KDC_ERR_PADATA_TYPE_NOSUPP,
+   N_("KDC has no support for padata type")},
+  {SHISHI_KDC_ERR_TRTYPE_NOSUPP,
+   N_("KDC has no support for transited type")},
+  {SHISHI_KDC_ERR_CLIENT_REVOKED,
+   N_("Clients credentials have been revoked")},
   {SHISHI_KDC_ERR_SERVICE_REVOKED,
-   "Credentials for server have been revoked"},
-  {SHISHI_KDC_ERR_TGT_REVOKED, "TGT has been revoked"},
-  {SHISHI_KDC_ERR_CLIENT_NOTYET, "Client not yet valid - try again later"},
-  {SHISHI_KDC_ERR_SERVICE_NOTYET, "Server not yet valid - try again later"},
-  {SHISHI_KDC_ERR_KEY_EXPIRED, "Password has expired "},
+   N_("Credentials for server have been revoked")},
+  {SHISHI_KDC_ERR_TGT_REVOKED,
+   N_("TGT has been revoked")},
+  {SHISHI_KDC_ERR_CLIENT_NOTYET,
+   N_("Client not yet valid - try again later")},
+  {SHISHI_KDC_ERR_SERVICE_NOTYET,
+   N_("Server not yet valid - try again later")},
+  {SHISHI_KDC_ERR_KEY_EXPIRED,
+   N_("Password has expired ")},
   {SHISHI_KDC_ERR_PREAUTH_FAILED,
-   "Pre-authentication information was invalid"},
+   N_("Pre-authentication information was invalid")},
   {SHISHI_KDC_ERR_PREAUTH_REQUIRED,
-   "Additional pre-authenticationrequired [40]"},
-  {SHISHI_KDC_ERR_SERVER_NOMATCH, "Requested server and ticket don't match"},
+   N_("Additional pre-authentication required")},
+  {SHISHI_KDC_ERR_SERVER_NOMATCH,
+   N_("Requested server and ticket don't match")},
   {SHISHI_KDC_ERR_MUST_USE_USER2USER,
-   "Server principal valid for user2user only"},
-  {SHISHI_KDC_ERR_PATH_NOT_ACCPETED, "KDC Policy rejects transited path"},
-  {SHISHI_KDC_ERR_SVC_UNAVAILABLE, "A service is not available"},
+   N_("Server principal valid for user2user only")},
+  {SHISHI_KDC_ERR_PATH_NOT_ACCPETED,
+   N_("KDC Policy rejects transited path")},
+  {SHISHI_KDC_ERR_SVC_UNAVAILABLE,
+   N_("A service is not available")},
   {SHISHI_KRB_AP_ERR_BAD_INTEGRITY,
-   "Integrity check on decrypted field failed"},
-  {SHISHI_KRB_AP_ERR_TKT_EXPIRED, "Ticket expired"},
-  {SHISHI_KRB_AP_ERR_TKT_NYV, "Ticket not yet valid"},
-  {SHISHI_KRB_AP_ERR_REPEAT, "Request is a replay"},
-  {SHISHI_KRB_AP_ERR_NOT_US, "The ticket isn't for us"},
-  {SHISHI_KRB_AP_ERR_BADMATCH, "Ticket and authenticator don't match"},
-  {SHISHI_KRB_AP_ERR_SKEW, "Clock skew too great"},
-  {SHISHI_KRB_AP_ERR_BADADDR, "Incorrect net address"},
-  {SHISHI_KRB_AP_ERR_BADVERSION, "Protocol version mismatch"},
-  {SHISHI_KRB_AP_ERR_MSG_TYPE, "Invalid msg type"},
-  {SHISHI_KRB_AP_ERR_MODIFIED, "Message stream modified"},
-  {SHISHI_KRB_AP_ERR_BADORDER, "Message out of order"},
-  {SHISHI_KRB_AP_ERR_BADKEYVER, "Specified version of key is not available"},
-  {SHISHI_KRB_AP_ERR_NOKEY, "Service key not available"},
-  {SHISHI_KRB_AP_ERR_MUT_FAIL, "Mutual authentication failed"},
-  {SHISHI_KRB_AP_ERR_BADDIRECTION, "Incorrect message direction"},
-  {SHISHI_KRB_AP_ERR_METHOD, "Alternative authentication method required"},
-  {SHISHI_KRB_AP_ERR_BADSEQ, "Incorrect sequence number in message"},
+   N_("Integrity check on decrypted field failed")},
+  {SHISHI_KRB_AP_ERR_TKT_EXPIRED,
+   N_("Ticket expired")},
+  {SHISHI_KRB_AP_ERR_TKT_NYV,
+   N_("Ticket not yet valid")},
+  {SHISHI_KRB_AP_ERR_REPEAT,
+   N_("Request is a replay")},
+  {SHISHI_KRB_AP_ERR_NOT_US,
+   N_("The ticket isn't for us")},
+  {SHISHI_KRB_AP_ERR_BADMATCH,
+   N_("Ticket and authenticator don't match")},
+  {SHISHI_KRB_AP_ERR_SKEW,
+   N_("Clock skew too great")},
+  {SHISHI_KRB_AP_ERR_BADADDR,
+   N_("Incorrect net address")},
+  {SHISHI_KRB_AP_ERR_BADVERSION,
+   N_("Protocol version mismatch")},
+  {SHISHI_KRB_AP_ERR_MSG_TYPE,
+   N_("Invalid msg type")},
+  {SHISHI_KRB_AP_ERR_MODIFIED,
+   N_("Message stream modified")},
+  {SHISHI_KRB_AP_ERR_BADORDER,
+   N_("Message out of order")},
+  {SHISHI_KRB_AP_ERR_BADKEYVER,
+   N_("Specified version of key is not available")},
+  {SHISHI_KRB_AP_ERR_NOKEY,
+   N_("Service key not available")},
+  {SHISHI_KRB_AP_ERR_MUT_FAIL,
+   N_("Mutual authentication failed")},
+  {SHISHI_KRB_AP_ERR_BADDIRECTION,
+   N_("Incorrect message direction")},
+  {SHISHI_KRB_AP_ERR_METHOD,
+   N_("Alternative authentication method required")},
+  {SHISHI_KRB_AP_ERR_BADSEQ,
+   N_("Incorrect sequence number in message")},
   {SHISHI_KRB_AP_ERR_INAPP_CKSUM,
-   "Inappropriate type of checksum in message"},
-  {SHISHI_KRB_AP_PATH_NOT_ACCEPTED, "Policy rejects transited path"},
+   N_("Inappropriate type of checksum in message")},
+  {SHISHI_KRB_AP_PATH_NOT_ACCEPTED,
+   N_("Policy rejects transited path")},
   {SHISHI_KRB_ERR_RESPONSE_TOO_BIG,
-   "Response too big for UDP, retry with TCP"},
-  {SHISHI_KRB_ERR_GENERIC, "Generic error (description in e-text)"},
-  {SHISHI_KRB_ERR_FIELD_TOOLONG, "Field is too long for this implementation"},
-  {SHISHI_KDC_ERROR_CLIENT_NOT_TRUSTED, "(pkinit)"},
-  {SHISHI_KDC_ERROR_KDC_NOT_TRUSTED, "(pkinit)"},
-  {SHISHI_KDC_ERROR_INVALID_SIG, "(pkinit)"},
-  {SHISHI_KDC_ERR_KEY_TOO_WEAK, "(pkinit)"},
-  {SHISHI_KDC_ERR_CERTIFICATE_MISMATCH, "(pkinit)"},
-  {SHISHI_KRB_AP_ERR_NO_TGT, "(user-to-user)"},
-  {SHISHI_KDC_ERR_WRONG_REALM, "(user-to-user)"},
-  {SHISHI_KRB_AP_ERR_USER_TO_USER_REQUIRED, "(user-to-user)"},
-  {SHISHI_KDC_ERR_CANT_VERIFY_CERTIFICATE, "(pkinit)"},
-  {SHISHI_KDC_ERR_INVALID_CERTIFICATE, "(pkinit)"},
-  {SHISHI_KDC_ERR_REVOKED_CERTIFICATE, "(pkinit)"},
-  {SHISHI_KDC_ERR_REVOCATION_STATUS_UNKNOWN, "(pkinit)"},
-  {SHISHI_KDC_ERR_REVOCATION_STATUS_UNAVAILABLE, "(pkinit)"},
-  {SHISHI_KDC_ERR_CLIENT_NAME_MISMATCH, "(pkinit)"},
-  {SHISHI_KDC_ERR_KDC_NAME_MISMATCH, "(pkinit)"}
+   N_("Response too big for UDP, retry with TCP")},
+  {SHISHI_KRB_ERR_GENERIC,
+   N_("Generic error (description in e-text)")},
+  {SHISHI_KRB_ERR_FIELD_TOOLONG,
+   N_("Field is too long for this implementation")},
+  {SHISHI_KDC_ERROR_CLIENT_NOT_TRUSTED,
+   N_("(pkinit)")},
+  {SHISHI_KDC_ERROR_KDC_NOT_TRUSTED,
+   N_("(pkinit)")},
+  {SHISHI_KDC_ERROR_INVALID_SIG,
+   N_("(pkinit)")},
+  {SHISHI_KDC_ERR_KEY_TOO_WEAK,
+   N_("(pkinit)")},
+  {SHISHI_KDC_ERR_CERTIFICATE_MISMATCH,
+   N_("(pkinit)")},
+  {SHISHI_KRB_AP_ERR_NO_TGT,
+   N_("(user-to-user)")},
+  {SHISHI_KDC_ERR_WRONG_REALM,
+   N_("(user-to-user)")},
+  {SHISHI_KRB_AP_ERR_USER_TO_USER_REQUIRED,
+   N_("(user-to-user)")},
+  {SHISHI_KDC_ERR_CANT_VERIFY_CERTIFICATE,
+   N_("(pkinit)")},
+  {SHISHI_KDC_ERR_INVALID_CERTIFICATE,
+   N_("(pkinit)")},
+  {SHISHI_KDC_ERR_REVOKED_CERTIFICATE,
+   N_("(pkinit)")},
+  {SHISHI_KDC_ERR_REVOCATION_STATUS_UNKNOWN,
+   N_("(pkinit)")},
+  {SHISHI_KDC_ERR_REVOCATION_STATUS_UNAVAILABLE,
+   N_("(pkinit)")},
+  {SHISHI_KDC_ERR_CLIENT_NAME_MISMATCH,
+   N_("(pkinit)")},
+  {SHISHI_KDC_ERR_KDC_NAME_MISMATCH,
+   N_("(pkinit)")}
 };
 
 /**
