@@ -1,5 +1,5 @@
 /* principal.c --- Get and set default principal.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2006  Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -33,15 +33,17 @@
 char *
 shishi_principal_default_guess (void)
 {
+#if HAVE_GETPWUID
   uid_t uid;
   struct passwd *pw;
 
   uid = getuid ();
   pw = getpwuid (uid);
 
-  if (pw)
+  if (pw && pw->pw_name)
     return xstrdup (pw->pw_name);
   else
+#endif
     return xstrdup ("user");
 }
 
@@ -422,6 +424,47 @@ shishi_principal_set (Shishi * handle,
   return SHISHI_OK;
 }
 
+/**
+ * shishi_derive_default_salt:
+ * @handle: shishi handle as allocated by shishi_init().
+ * @name: principal name of user.
+ * @salt: output variable with newly allocated salt string.
+ *
+ * Derive the default salt from a principal.  The default salt is the
+ * concatenation of the decoded realm and principal.
+ *
+ * Return value: Return SHISHI_OK if successful.
+ **/
+int
+shishi_derive_default_salt (Shishi * handle,
+			    const char *name,
+			    char **salt)
+{
+  char *principal;
+  char *realm;
+  int rc;
+
+  rc = shishi_parse_name (handle, name, &principal, &realm);
+  if (rc != SHISHI_OK)
+    return rc;
+
+  if (!principal || !realm)
+    {
+      if (realm)
+	free (realm);
+      if (principal)
+	free (principal);
+      return SHISHI_INVALID_PRINCIPAL_NAME;
+    }
+
+  *salt = xasprintf ("%s%s", realm, principal);
+
+  free (realm);
+  free (principal);
+
+  return SHISHI_OK;
+}
+
 /**
  * shishi_server_for_local_service:
  * @handle: shishi handle as allocated by shishi_init().
