@@ -1,5 +1,5 @@
 /* shishi.h --- Header file for Shishi library.                       -*- c -*-
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2006  Simon Josefsson
  *
  * This file is part of Shishi.
  *
@@ -27,7 +27,7 @@
 #include <time.h>		/* time_t */
 #include <shishi-int.h>		/* uint32_t */
 
-#define SHISHI_VERSION "0.0.22"
+#define SHISHI_VERSION "0.0.23"
 
 /* Error codes */
 typedef enum
@@ -83,7 +83,8 @@ typedef enum
   SHISHI_PRIV_BAD_KEYTYPE = 48,
   SHISHI_FILE_ERROR = 49,
   SHISHI_ENCAPREPPART_BAD_KEYTYPE = 50,
-  SHISHI_LAST_ERROR = 50
+  SHISHI_GETTIMEOFDAY_ERROR = 51,
+  SHISHI_LAST_ERROR = 51
 }
 Shishi_rc;
 
@@ -680,6 +681,9 @@ extern int shishi_principal_set (Shishi * handle,
 				 const char *namefield, const char *name);
 extern int shishi_parse_name (Shishi * handle, const char *name,
 			      char **principal, char **realm);
+extern int shishi_derive_default_salt (Shishi * handle,
+				       const char *name,
+				       char **salt);
 extern char *shishi_server_for_local_service (Shishi * handle,
 					      const char *service);
 
@@ -1804,6 +1808,12 @@ extern int shishi_key_from_string (Shishi * handle,
 				   const char *salt, size_t saltlen,
 				   const char *parameter,
 				   Shishi_key ** outkey);
+extern int shishi_key_from_name (Shishi * handle,
+				 int32_t type,
+				 const char *name,
+				 const char *password, size_t passwordlen,
+				 const char *parameter,
+				 Shishi_key ** outkey);
 
 /* keys.c */
 extern Shishi_key *shishi_keys_for_serverrealm_in_file (Shishi * handle,
